from django.db import transaction
try:
    atomic_transaction = transaction.atomic
except AttributeError:
    transaction.atomic = transaction.commit_on_success
try:
    from django.db.transaction import atomic
except ImportError:
    from django.db.transaction import commit_on_success as atomic


@atomic
def decorator_reset_choice(choice):
    choice.votes = 0
    choice.save()
    return choice.pk


@atomic()
def callable_reset_choice(choice):
    choice.votes = 0
    choice.save()
    return choice.pk


def ctxt_man_reset_choice(choice):
    with atomic():
        choice.votes = 0
        choice.save()
        return choice.pk


@transaction.atomic
def transaction_decorator_reset_choice(choice):
    choice.votes = 0
    choice.save()
    return choice.pk


@transaction.atomic()
def transaction_callable_reset_choice(choice):
    choice.votes = 0
    choice.save()
    return choice.pk


def transaction_ctxt_man_reset_choice(choice):
    with transaction.atomic():
        choice.votes = 0
        choice.save()
        return choice.pk



# Django settings for 'project' project.

import os
PROJECT_PATH = os.path.abspath(os.path.split(__file__)[0])

DEBUG = True
TEMPLATE_DEBUG = DEBUG

ADMINS = (
    # ('Your Name', 'your_email@domain.com'),
)

MANAGERS = ADMINS

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(PROJECT_PATH, 'zoo.db'),
        'USER': '',
        'PASSWORD': '',
        'HOST': '',
        'PORT': '',
    }
}

# Local time zone for this installation. Choices can be found here:
# http://en.wikipedia.org/wiki/List_of_tz_zones_by_name
# although not all choices may be available on all operating systems.
# If running in a Windows environment this must be set to the same as your
# system time zone.
TIME_ZONE = 'America/Chicago'

# Language code for this installation. All choices can be found here:
# http://www.i18nguy.com/unicode/language-identifiers.html
LANGUAGE_CODE = 'en-us'

SITE_ID = 1

# If you set this to False, Django will make some optimizations so as not
# to load the internationalization machinery.
USE_I18N = True

# Absolute path to the directory that holds media.
# Example: "/home/media/media.lawrence.com/"
MEDIA_ROOT = ''

# URL that handles the media served from MEDIA_ROOT. Make sure to use a
# trailing slash if there is a path component (optional in other cases).
# Examples: "http://media.lawrence.com", "http://example.com/media/"
MEDIA_URL = ''

# URL prefix for admin media -- CSS, JavaScript and images. Make sure to use a
# trailing slash.
# Examples: "http://foo.com/media/", "/media/".
ADMIN_MEDIA_PREFIX = '/media/'

# Make this unique, and don't share it with anybody.
SECRET_KEY = '3r+%mj&(wg@l2wcq)&g!x^cj2@u28vr=pahbive*#am44-p$u)'

# List of callables that know how to import templates from various sources.
TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.load_template_source',
    'django.template.loaders.app_directories.load_template_source',
#     'django.template.loaders.eggs.load_template_source',
)

MIDDLEWARE_CLASSES = (
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.middleware.doc.XViewMiddleware',
)

ROOT_URLCONF = 'project.urls'

TEMPLATE_DIRS = (
    os.path.join(PROJECT_PATH, 'templates'),
    # Put strings here, like "/home/html/django_templates" or "C:/www/django/templates".
    # Always use forward slashes, even on Windows.
    # Don't forget to use absolute paths, not relative paths.
)

INSTALLED_APPS = (
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'project.zoo'
)
handler500 = 'django.views.defaults.server_error'




from importlib import import_module

from nose.plugins.skip import SkipTest

import django
from django.test import TestCase

from nosedjangotests.polls.models import Choice


class AtomicTestCase(TestCase):
    use_transaction_isolation = True

    fixtures = [
        'polls1.json',
        'choices.json',
    ]

    def test_decorator_atomic(self):
        if django.VERSION < (1, 7):
            raise SkipTest('This test is only an issue in django 1.8+')

        choice = Choice.objects.get()
        choice.votes = 42
        tester = import_module("polls.helpers")
        choice_pk = tester.decorator_reset_choice(choice=choice)
        self.assertEqual(choice_pk, choice.pk)
        choice = Choice.objects.get()
        self.assertEqual(choice.votes, 0)

    def test_callable_decorator_atomic(self):
        if django.VERSION < (1, 7):
            raise SkipTest('This test is only an issue in django 1.8+')

        choice = Choice.objects.get()
        choice.votes = 42
        tester = import_module("polls.helpers")
        choice_pk = tester.callable_reset_choice(choice=choice)
        self.assertEqual(choice_pk, choice.pk)
        choice = Choice.objects.get()
        self.assertEqual(choice.votes, 0)

    def test_context_manager_atomic(self):
        if django.VERSION < (1, 7):
            raise SkipTest('This test is only an issue in django 1.8+')

        choice = Choice.objects.get()
        choice.votes = 42
        tester = import_module("polls.helpers")
        choice_pk = tester.ctxt_man_reset_choice(choice=choice)
        self.assertEqual(choice_pk, choice.pk)
        choice = Choice.objects.get()
        self.assertEqual(choice.votes, 0)


class TransactionTestCase(TestCase):
    use_transaction_isolation = True

    fixtures = [
        'polls1.json',
        'choices.json',
    ]

    def test_decorator_atomic(self):
        if django.VERSION < (1, 7):
            raise SkipTest('This test is only an issue in django 1.8+')

        choice = Choice.objects.get()
        choice.votes = 42
        tester = import_module("polls.helpers")
        choice_pk = tester.transaction_decorator_reset_choice(choice=choice)
        self.assertEqual(choice_pk, choice.pk)
        choice = Choice.objects.get()
        self.assertEqual(choice.votes, 0)

    def test_callable_decorator_atomic(self):
        if django.VERSION < (1, 7):
            raise SkipTest('This test is only an issue in django 1.8+')

        choice = Choice.objects.get()
        choice.votes = 42
        tester = import_module("polls.helpers")
        choice_pk = tester.transaction_callable_reset_choice(choice=choice)
        self.assertEqual(choice_pk, choice.pk)
        choice = Choice.objects.get()
        self.assertEqual(choice.votes, 0)

    def test_context_manager_atomic(self):
        if django.VERSION < (1, 7):
            raise SkipTest('This test is only an issue in django 1.8+')

        choice = Choice.objects.get()
        choice.votes = 42
        tester = import_module("polls.helpers")
        choice_pk = tester.transaction_ctxt_man_reset_choice(choice=choice)
        self.assertEqual(choice_pk, choice.pk)
        choice = Choice.objects.get()
        self.assertEqual(choice.votes, 0)



from unittest import TestCase
from django.conf import settings


DEFAULT_ROOT_URLCONF = settings.ROOT_URLCONF


class ACustomURLConfTestCase(TestCase):
    def test_root_urlconf_is_default(self):
        from django.conf import settings
        self.assertEqual(settings.ROOT_URLCONF, DEFAULT_ROOT_URLCONF)


class BCustomURLConfTestCase(TestCase):
    urls = 'foo'

    def test_root_urlconf_is_set(self):
        from django.conf import settings
        self.assertEqual(settings.ROOT_URLCONF, 'foo')


class CCustomURLConfTestCase(TestCase):
    def test_root_urlconf_is_default(self):
        from django.conf import settings
        self.assertEqual(settings.ROOT_URLCONF, DEFAULT_ROOT_URLCONF)


class DCustomURLConfTestCase(TestCase):
    urls = 'bar'

    def test_root_urlconf_is_set(self):
        from django.conf import settings
        self.assertEqual(settings.ROOT_URLCONF, 'bar')



from django.test import TransactionTestCase

import tempfile


class SeleniumTestCase(TransactionTestCase):

    def test_simple(self):
        driver = self.driver
        driver.get('http://www.google.com')
        firefox_profile = self.driver.firefox_profile

        self.assertEqual(
            firefox_profile.default_preferences['browser.download.dir'],
            tempfile.gettempdir()
        )

        self.assertEqual(
            firefox_profile.default_preferences['browser.helperApps.neverAsk.saveToDisk'],
            'application/ourfakemimetype'
        )

        self.assertEqual(
            firefox_profile.default_preferences['browser.download.folderList'],
            2
        )

        self.assertEqual(
            firefox_profile.default_preferences['browser.download.manager.showWhenStarting'],
            False
        )





from datetime import datetime

from django.db import transaction
from django.test.testcases import TestCase, TransactionTestCase

from nosedjangotests.polls.models import Poll


class MakePollMixin():
    def make_poll(self):
        Poll.objects.create(
            question='Test',
            pub_date=datetime.now(),
        )

    def make_poll_and_error(self):
        Poll.objects.create(
            question='Test',
            pub_date=datetime.now(),
        )
        raise Exception("Oops, that is not a question")


class TestTestCase(TestCase, MakePollMixin):
    def test_make_poll(self):
        with transaction.atomic():
            self.make_poll()
        self.assertEqual(Poll.objects.count(), 1)

    def test_make_poll_and_error(self):
        with self.assertRaisesMessage(Exception, "Oops"):
            with transaction.atomic():
                self.make_poll_and_error()
        self.assertEqual(Poll.objects.count(), 1)


class TestTransactionTestCase(TransactionTestCase, MakePollMixin):
    def test_make_poll(self):
        with transaction.atomic():
            self.make_poll()
        self.assertEqual(Poll.objects.count(), 1)

    def test_make_poll_and_error(self):
        with self.assertRaisesMessage(Exception, "Oops"):
            with transaction.atomic():
                self.make_poll_and_error()
        self.assertEqual(Poll.objects.count(), 0)
